import logging
import re

from .BooleanSchemaOption import BooleanSchemaOption
from .SchemaOptionDefs import SchemaOptionDefs


def citep_transform(content: str):
    logging.info("Transform: citep_transform")
    return re.sub(r"\\citep{", r"\\cite{", content)


class NatbibSchemaOption(BooleanSchemaOption):
    def __init__(self):
        super(NatbibSchemaOption, self).__init__()

        self.add(
            True,
            SchemaOptionDefs(packages=["package-natbib.def"]),
        )

        self.add(False, SchemaOptionDefs(transforms=[citep_transform]))

        self.set_default(False)



import logging
import os
from typing import Callable, Dict, List, Union

import pkg_resources

from .options import (
    AsideSchemaOption,
    BibstyleSchemaOption,
    BooleanSchemaOption,
    CalloutSchemaOption,
    CitestyleSchemaOption,
    CodeSchemaOption,
    CustomTemplateDefs,
    HideLinksSchemaOption,
    NatbibSchemaOption,
    RaggedBottomSchemaOption,
    SchemaOptionDefs,
    SloppySchemaOption,
    StringSchemaOption,
)
from .TemplateOptions import TemplateOptions
from .utils import just_log_errors, log_and_raise_errors


def get_def_file_path(filename: str):
    def_path = pkg_resources.resource_filename("curvenote_template", "defs")
    return os.path.join(def_path, filename)


def get_def_template_file_path(template_location: str, filename: str):
    return os.path.join(template_location, filename)


@just_log_errors(lambda *args: f"Could not open def file at {args[0]}, skipping...")
def get_def_from_pkg(filename: str):
    with open(get_def_file_path(filename), "r") as file:
        return file.read()


@just_log_errors(lambda *args: f"Could not open def file at {args[0]}, skipping...")
def read_from_file(fullpath: str):
    with open(fullpath, "r") as file:
        return file.read()


class DefBuilder:
    def __init__(self):
        """
        All schema options should be registered here
        """
        self.schema_options: Dict[
            str, Union[StringSchemaOption, BooleanSchemaOption]
        ] = {}
        self.defs_path: str = ""

        self.schema_options["aside"] = AsideSchemaOption()
        self.schema_options["callout"] = CalloutSchemaOption()
        self.schema_options["code"] = CodeSchemaOption()
        self.schema_options["raggedbottom"] = RaggedBottomSchemaOption()
        self.schema_options["sloppy"] = SloppySchemaOption()
        self.schema_options["hidelinks"] = HideLinksSchemaOption()
        self.schema_options["natbib"] = NatbibSchemaOption()
        self.schema_options["bibstyle"] = BibstyleSchemaOption()
        self.schema_options["citestyle"] = CitestyleSchemaOption()

    def build(self, options: TemplateOptions, target_path: str):
        def_paths = self.compose(options)
        def_contents = self.serialize(*def_paths)
        self.write(target_path, *def_contents)

    def _resolve_options(
        self, template_options: TemplateOptions
    ) -> List[Union[SchemaOptionDefs, CustomTemplateDefs]]:
        # iterate over registered options, check to see if the
        # option has been set in the template options, if not
        # use the default
        defs_list: List[Union[SchemaOptionDefs, CustomTemplateDefs]] = []
        logging.info("Iterating over %s options", len(self.schema_options))
        for name, schema_option in self.schema_options.items():
            logging.info("Checking option %s", name)
            if (
                template_options.schema_options
                and name in template_options.schema_options
            ):
                template_setting = template_options.schema_options[name]
                defs = schema_option.get(template_setting)
                logging.info("%s - using template option: %s", name, template_setting)
            else:
                defs = schema_option.get()  # default
                logging.info(
                    "%s - using schema default option: %s", name, schema_option.default
                )
            defs_list.append(defs)

        return defs_list

    def compose(self, template_options: TemplateOptions):
        logging.info("DefBuilder.compose")
        passopts_paths = []
        packages_paths = []
        setup_paths = []

        list_of_defs = self._resolve_options(template_options)

        for defs in list_of_defs:
            if len(defs.passopts) > 0:
                for passopt in defs.passopts:
                    passopts_paths.append(get_def_file_path(passopt))
            if len(defs.packages) > 0:
                for package in defs.packages:
                    packages_paths.append(get_def_file_path(package))
            if len(defs.setup) > 0:
                for setup in defs.setup:
                    full_path = (
                        get_def_template_file_path(
                            template_options.template_location, setup
                        )
                        if defs.in_template
                        else get_def_file_path(setup)
                    )
                    setup_paths.append(full_path)

        # dedupe - don't care about ordering
        passopts_paths = list(set(passopts_paths))
        packages_paths = list(set(packages_paths))
        setup_paths = list(set(setup_paths))

        logging.info("passopts %s\n", passopts_paths)
        logging.info("packages %s\n", packages_paths)
        logging.info("setup %s\n", setup_paths)

        # log any option specified in the template that are
        # not supported by a registered schema option here
        for name, _ in template_options.schema_options.items():
            if name not in self.schema_options:
                logging.warning("Unknown schema option %s, skipping...", name)

        return passopts_paths, packages_paths, setup_paths

    def get_content_transforms(
        self, template_options: TemplateOptions
    ) -> List[Callable[[str], str]]:
        list_of_defs = self._resolve_options(template_options)
        list_of_transforms: List[Callable[[str], str]] = []
        for defs in list_of_defs:
            if len(defs.transforms) > 0:
                list_of_transforms += defs.transforms
        return list_of_transforms

    def serialize(self, passopts_paths, packages_paths, setup_paths):
        """
        Load defs from files and concatenate
        """

        # base configuration
        base_passopts = get_def_from_pkg("passopts-base.def")
        base_packages = get_def_from_pkg("package-base.def")
        base_setup = get_def_from_pkg("setup-base.def")

        # read contents from def files
        passopts_paths = [read_from_file(p) for p in passopts_paths]
        packages_paths = [read_from_file(p) for p in packages_paths]
        setup_paths = [read_from_file(s) for s in setup_paths]

        newline = "\n"
        passopts = f"% Pass Options Section\n% base\n{base_passopts}\n% template\n{newline.join(passopts_paths)}\n"
        packages = f"% Use Package Section\n% base\n{base_packages}\n% template\n{newline.join(packages_paths)}\n"
        setup = (
            "% Setup Section\n"
            f"% base\n{base_setup}\n"
            f"% template\n{newline.join(setup_paths)}\n"
        )

        return passopts, packages, setup

    @log_and_raise_errors(lambda *args: f"Could not write defs to: {args[0]}")
    def write(self, target_path: str, passopts: str, packages: str, setup: str):
        with open(
            os.path.join(target_path, "curvenote.passopts.def"), "w"
        ) as defs_file:
            defs_file.write(passopts)
        with open(
            os.path.join(target_path, "curvenote.packages.def"), "w"
        ) as defs_file:
            defs_file.write(packages)
        with open(os.path.join(target_path, "curvenote.setup.def"), "w") as defs_file:
            defs_file.write(setup)
        with open(os.path.join(target_path, "curvenote.def"), "w") as defs_file:
            defs_file.write(
                "% Start Curvenote Definitions\n\\input{curvenote.passopts.def}\n"
                "\\input{curvenote.packages.def}\n\\input{curvenote.setup.def}\n% End Curvenote Definitions\n"
            )



import logging
import os
from pathlib import Path

import typer

from ..TemplateLoader import TemplateLoader
from ..TemplateOptions import TemplateOptions


def validate(
    template_path: Path = typer.Argument(
        ...,
        help=("Local folder containing the Curvenote compatible template to validate"),
        exists=True,
        dir_okay=True,
        file_okay=False,
        resolve_path=True,
    )
):
    try:
        TemplateLoader.validate(str(template_path))
    except ValueError as err:
        raise typer.Exit(code=1)
    raise typer.Exit(code=0)



from typing import Dict, Optional, Union

from .SchemaOptionDefs import CustomTemplateDefs, SchemaOptionDefs


class SchemaOption:
    def add(self, name: str, value: SchemaOptionDefs):
        raise TypeError("Implement in derived class")

    def set_default(self, name: str):
        raise TypeError("Implement in derived class")

    def get(
        self, name: Optional[str] = None
    ) -> Union[SchemaOptionDefs, CustomTemplateDefs]:
        raise TypeError("Implement in derived class")

    def names(self):
        raise TypeError("Implement in derived class")



import glob
import logging
import os
from pathlib import Path, PurePath
from shutil import copyfile

import typer
import yaml

from .. import DocModel, LatexBuilder, TemplateLoader


def build(
    target_folder: Path = typer.Argument(
        ...,
        help=(
            "Local folder in which to construct the Latex assets. If TARGET exists it"
            "and all files will be removed and a new empty folder structure created"
        ),
        resolve_path=True,
        dir_okay=True,
        file_okay=False,
    ),
    content_path: Path = typer.Argument(
        ...,
        help=(
            "Path to a folder with containing content to render. Folder shoud contain the following files:\n"
            "  - main.tex\n"
            "  - main.bib\n"
            "  - data.yaml\n"
            "Along with any additional graphics assets"
        ),
        exists=True,
        dir_okay=True,
        file_okay=False,
        resolve_path=True,
    ),
    template_path: Path = typer.Argument(
        None,
        help=(
            "Path to a Curvenote compatible LaTeX template folder."
            "This is intended for use with local Curvenote templates or in template development."
            "Omitting this option will use the built in template."
        ),
        exists=True,
        dir_okay=True,
        file_okay=False,
        resolve_path=True,
    ),
    lipsum: bool = typer.Option(
        False,
        help=(
            "If specified will patch the document with '\\usepackage{lipsum}'."
            "For use in template testing where `example/content.tex` uses the lipsum package."
        ),
    ),
    strict: bool = typer.Option(
        False,
        help=(
            "If true, then missing required tagged content or options will halt the process."
        ),
    ),
    no_copy: bool = typer.Option(
        False,
        help=("If true, then image assets will not be copied into the target folder."),
    ),
):
    typer.echo(f"Target folder: {target_folder}")

    typer.echo(f"Content path: {content_path}")
    content_file = Path(content_path, "main.tex")
    if content_file.exists() and content_file.is_file():
        typer.echo("Found main.tex")
    else:
        typer.echo("main.tex not found")
        raise typer.Exit(code=1)
    data_file = Path(content_path, "data.yml")
    if data_file.exists() and data_file.is_file():
        typer.echo("Found data.yml")
    else:
        typer.echo("data.yml not found")
        raise typer.Exit(code=1)

    bib_file = Path(content_path, "main.bib")
    if bib_file.exists():
        typer.echo("Found main.bib")

    if template_path:
        typer.echo(f"Using template at: {template_path}")
    else:
        typer.echo("Using built in template")

    if lipsum:
        typer.echo(f"Adding lipsum package to final document")

    content = ""
    try:
        with open(content_file) as cfile:
            content = cfile.read()
        typer.echo("Loaded content")
    except:
        typer.echo("Could not read content")
        raise typer.Exit(code=1)

    docmodel = {}
    try:
        with open(data_file) as dfile:
            docmodel = yaml.load(dfile.read(), Loader=yaml.FullLoader)
        typer.echo("Loaded data")
    except:
        typer.echo("Could not load data (DocModel)")
        raise typer.Exit(code=1)

    if lipsum:
        docmodel["lipsum"] = True

    loader = TemplateLoader(str(target_folder))
    if template_path:
        template_options, renderer = loader.initialise_from_path(str(template_path))
    else:
        template_options, renderer = loader.initialise_with_builtin_template()
    typer.echo("Template loaded")

    builder = LatexBuilder(template_options, renderer, str(target_folder))
    builder.build(DocModel(docmodel), [content], bibtex=None, raise_if_invalid=strict)

    if bib_file.exists():
        copyfile(bib_file, os.path.join(str(target_folder), "main.bib"))

    typer.echo("Checking content_path for image assets")
    typer.echo(f"Content Path: {content_path}")
    typer.echo(f"Target Folder: {target_folder}")
    if no_copy:
        typer.echo("--no-copy option is set - not copying image assets")
    else:
        image_types = ["*.png", "*.jpg", "*.jpeg", "*.eps", "*.gif", "*.bmp"]
        image_files = []
        for im_type in image_types:
            image_files.extend(
                glob.glob(f"{content_path}/**/{im_type}", recursive=True)
            )
        if len(image_files) > 0:
            typer.echo(f"Found {len(image_files)} image assets")
            for im_file_path in image_files:
                src_path, filename = os.path.split(im_file_path)
                internal_src_path = Path(src_path).relative_to(content_path)
                os.makedirs(target_folder / internal_src_path, exist_ok=True)
                dest = target_folder / internal_src_path / filename
                copyfile(im_file_path, dest)
                typer.echo(f"Copied {filename} to {dest}")
        else:
            typer.echo("No image assets found")

    typer.echo("Done!")



from .SchemaOptionDefs import CustomTemplateDefs, SchemaOptionDefs
from .StringSchemaOption import StringSchemaOption


class CodeSchemaOption(StringSchemaOption):
    def __init__(self):
        super(CodeSchemaOption, self).__init__()

        self.add("framed", SchemaOptionDefs(setup=["setup-code-framed.def"]))

        self.add(
            "verbatim",
            SchemaOptionDefs(
                packages=["package-fancyvrb.def"], setup=["setup-code-verbatim.def"]
            ),
        )

        self.add(
            "highlight",
            SchemaOptionDefs(
                packages=["package-fancyvrb.def"], setup=["setup-code-verbatim.def"]
            ),
        )

        # TODO needs a fix or alternative
        # self.add("minted", SchemaOptionDefs(
        #     passopts=['passopts-code-xcolor.def'],
        #     packages=['package-minted.def','package-xcolor.def'],
        #     setup=['setup-code-minted.def']
        # ))

        self.add("code.def", CustomTemplateDefs(setup=["code.def"]))

        self.set_default("verbatim")



from typing import Dict, Optional, Union

from .SchemaOption import SchemaOption
from .SchemaOptionDefs import CustomTemplateDefs, SchemaOptionDefs


class StringSchemaOption(SchemaOption):
    def __init__(self):
        self.value_map: Dict[str, Union[SchemaOptionDefs, CustomTemplateDefs]] = {}
        self.default: Union[str, None] = None

    def add(self, name: str, value: SchemaOptionDefs):
        if name in self.value_map:
            raise ValueError(f"Duplicate option {name}")
        self.value_map[name] = value

    def set_default(self, name: str):
        self.default = name
        self.value_map["default"] = self.value_map[name]

    def get(
        self, name: Optional[str] = None
    ) -> Union[SchemaOptionDefs, CustomTemplateDefs]:
        if name is None and self.default is None:
            raise ValueError("No default option set")
        if name is not None and name not in self.value_map:
            raise ValueError(f"Unknown option: {name}")
        if name in self.value_map:
            return self.value_map[name]
        return self.value_map[self.default]

    def names(self):
        return [k for k, v in self.value_map.items()]



from .SchemaOptionDefs import CustomTemplateDefs, SchemaOptionDefs
from .StringSchemaOption import StringSchemaOption


class AsideSchemaOption(StringSchemaOption):
    def __init__(self):
        super(AsideSchemaOption, self).__init__()

        self.add("marginpar", SchemaOptionDefs(setup=["setup-aside-marginpar.def"]))

        self.add("framed", SchemaOptionDefs(setup=["setup-aside-framed.def"]))

        self.add(
            "mdframed",
            SchemaOptionDefs(
                passopts=["passopts-mdframed.def"],
                packages=["package-mdframed.def"],
                setup=["setup-aside-mdframed.def"],
            ),
        )

        self.add(
            "callout",
            SchemaOptionDefs(
                passopts=["passopts-mdframed.def"],
                packages=["package-mdframed.def"],
                setup=["setup-aside-callout.def"],
            ),
        )

        self.add("aside.def", CustomTemplateDefs(setup=["aside.def"]))

        self.set_default("marginpar")



from typing import Dict, Optional, Union

from .SchemaOption import SchemaOption
from .SchemaOptionDefs import CustomTemplateDefs, SchemaOptionDefs


class BooleanSchemaOption(SchemaOption):
    def __init__(self):
        self.when_true: Union[SchemaOptionDefs, CustomTemplateDefs, None] = None
        self.when_false: Union[SchemaOptionDefs, CustomTemplateDefs, None] = None
        self.default: Union[bool, None] = None

    def add(self, flag: bool, value: SchemaOptionDefs):
        if flag:
            self.when_true = value
        else:
            self.when_false = value

    def set_default(self, flag: bool):
        self.default = flag

    def get(
        self, flag: Optional[bool] = None
    ) -> Union[SchemaOptionDefs, CustomTemplateDefs]:
        if flag is None and self.default is None:
            raise ValueError("No default option set")
        real_flag = self.default if flag is None else flag
        return self.when_true if real_flag else self.when_false

    def names(self):
        return []



from .SchemaOptionDefs import CustomTemplateDefs, SchemaOptionDefs
from .StringSchemaOption import StringSchemaOption


class CalloutSchemaOption(StringSchemaOption):
    def __init__(self):
        super(CalloutSchemaOption, self).__init__()

        self.add(
            "framed",
            SchemaOptionDefs(
                setup=["setup-callout-framed.def"],
            ),
        )

        self.add(
            "mdframed",
            SchemaOptionDefs(
                passopts=["passopts-mdframed.def"],
                packages=["package-mdframed.def"],
                setup=["setup-callout-mdframed.def"],
            ),
        )

        self.add(
            "callout",
            SchemaOptionDefs(
                passopts=["passopts-mdframed.def", "passopts-code-xcolor.def"],
                packages=["package-mdframed.def", "package-xcolor.def"],
                setup=["setup-callout.def"],
            ),
        )

        self.add("callout.def", CustomTemplateDefs(setup=["callout.def"]))

        self.set_default("framed")



import logging


def just_log_errors(message_func):
    """
    creates a decorator to allow us to eat errors and continue
    while providing a custom log message
    """

    def decorator(func):
        def inner(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except ValueError:
                logging.info(message_func(*args, **kwargs))

        return inner

    return decorator


def log_and_raise_errors(message_func):
    """
    creates a decorator to allow us to eat errors and continue
    while providing a custom log message
    """

    def decorator(func):
        def inner(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except ValueError as err:
                logging.error(message_func(*args, **kwargs))
                logging.error("Error: %s", str(err))
                raise err

        return inner

    return decorator



import logging
import sys

import typer

from ..version import __version__
from .build import build
from .build_lite import build_lite
from .validate import validate

logger = logging.getLogger()

logger.setLevel(logging.INFO)
handler = logging.StreamHandler(sys.stdout)
handler.setLevel(logging.INFO)
formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
handler.setFormatter(formatter)
logger.addHandler(handler)


app = typer.Typer()

app.command(help=("Validate a Curvenote LaTeX Template"))(validate)
app.command(
    help=(
        "Build a LaTeX document based on a free-form template, accompanying docmodel"
        "data structure and content. This can be used for general template rendering independent of"
        "Curvenote's prescriptive template structure."
        "To build based on (and to develop/test) Curvenote templates use `build`."
    )
)(build_lite)
app.command(
    help=(
        "Build a LaTeX document based on a Curvenote LaTeX Template, accompanying docmodel"
        "data structure and content."
        "Can be used to develop/test Curvenote templates."
    )
)(build)


def version_callback(value: bool):
    if value:
        typer.echo(
            r"""
                LaTeX Template Generating System by Curvenote
            """
        )
        typer.echo(f"Version: {__version__}")
        raise typer.Exit()


@app.callback()
def main(
    version: bool = typer.Option(
        None, "--version", callback=version_callback, is_eager=True
    )
):
    return



from .BooleanSchemaOption import BooleanSchemaOption
from .SchemaOptionDefs import SchemaOptionDefs


class HideLinksSchemaOption(BooleanSchemaOption):
    def __init__(self):
        super(HideLinksSchemaOption, self).__init__()

        self.add(True, SchemaOptionDefs(setup=["setup-hyperref-hidelinks.def"]))

        self.add(False, SchemaOptionDefs())

        self.set_default(False)



from typing import Callable, List, Optional


class SchemaOptionDefs:
    _passopts: List[str]
    _packages: List[str]
    _setup: List[str]
    _transforms: List[Callable[[str], str]]

    def __init__(
        self,
        passopts: List[str] = [],
        packages: List[str] = [],
        setup: List[str] = [],
        transforms: List[Callable[[str], str]] = [],
    ):
        self._passopts = passopts
        self._packages = packages
        self._setup = setup
        self._transforms = transforms

    @property
    def passopts(self):
        return self._passopts

    @property
    def packages(self):
        return self._packages

    @property
    def setup(self):
        return self._setup

    @property
    def transforms(self):
        return self._transforms

    @property
    def in_template(self):
        return False


class CustomTemplateDefs(SchemaOptionDefs):
    """
    This allow template creators to define custom options
    by adding .def files to the template directory.
    """

    def __init__(self, *args, **kwargs):
        super(CustomTemplateDefs, self).__init__(*args, **kwargs)

    @property
    def in_template(self):
        return True



from .BooleanSchemaOption import BooleanSchemaOption
from .SchemaOptionDefs import SchemaOptionDefs


class SloppySchemaOption(BooleanSchemaOption):
    def __init__(self):
        super(SloppySchemaOption, self).__init__()

        self.add(True, SchemaOptionDefs(setup=["cmd-sloppy.def"]))

        self.add(False, SchemaOptionDefs())

        self.set_default(False)



import logging
import re

from .BooleanSchemaOption import BooleanSchemaOption
from .SchemaOptionDefs import SchemaOptionDefs


class BibstyleSchemaOption(BooleanSchemaOption):
    """
    Enables/disables the default bibstyle
    """

    def __init__(self):
        super(BibstyleSchemaOption, self).__init__()

        self.add(
            True,
            SchemaOptionDefs(
                setup=["setup-bib-style.def"],
            ),
        )

        self.add(False, SchemaOptionDefs())

        self.set_default(False)



import logging
import os
import shutil
from typing import Any, Dict, List, NewType, Optional

from . import DefBuilder, DocModel, TemplateOptions, TemplateRenderer
from .utils import log_and_raise_errors

logger = logging.getLogger()


class LatexBuilder:
    def __init__(
        self, options: TemplateOptions, renderer: TemplateRenderer, target_folder: str
    ):
        self.options = options
        self.renderer = renderer
        self.target_folder = target_folder

    def validate(self, data: DocModel, raise_if_invalid):
        logging.info("Validating docmodel data...")
        required_options = [
            opt["id"]
            for opt in self.options.config_options
            if "required" in opt and opt["required"]
        ]
        missing_options = [
            r
            for r in required_options
            if "options" in data and r not in data["options"]
        ]
        logging.warn("Some REQUIRED user options are not provided: %s", missing_options)
        if len(missing_options) > 0 and raise_if_invalid:
            raise ValueError(
                "Some REQUIRED user options are not provided: %s" % missing_options
            )

    def build(
        self,
        data: DocModel,
        content: List[str],
        bibtex: Optional[str] = None,
        raise_if_invalid: bool = True,
    ):
        logging.info("Rendering template...")
        self.validate(data, raise_if_invalid)
        if self.renderer is None:
            logging.info(
                "TemplateRender is not available, TemplateLoader not initialized"
            )
            raise ValueError(
                "TemplateRender is not available, TemplateLoader not initialized"
            )

        data["CONTENT"] = content[0]
        rendered_content = [self.renderer.render(data)]
        self._write(rendered_content, bibtex)

    @log_and_raise_errors(lambda *args: "Could not write final document")
    def _write(self, content: List[str], bibtex: Optional[str]):
        logging.info("ProjectBuilder - writing...")

        def_builder = DefBuilder()
        def_builder.build(self.options, self.target_folder)
        content_transforms = def_builder.get_content_transforms(self.options)

        if not self.options.compact:
            raise NotImplementedError(
                "LatexBuilder not implemented for books layouts yet"
            )

        logging.info("Applying content transforms")
        transformed_content = ""
        for chunk in content:
            transformed_chunk = chunk
            for transform in content_transforms:
                transformed_chunk = transform(transformed_chunk)
            transformed_content += transformed_chunk + "\n"

        logging.info("Writing main.tex...")
        with open(os.path.join(self.target_folder, "main.tex"), "w+") as file:
            file.write(transformed_content)

        logging.info("Writing main.bib...")
        if bibtex:
            with open(os.path.join(self.target_folder, "main.bib"), "w+") as file:
                file.write(bibtex)

        logging.info("Cleaning up...")
        files = ["template.tex", "template.yml"]
        for file in files:
            if os.path.exists(os.path.join(self.target_folder, file)):
                shutil.move(
                    os.path.join(self.target_folder, file),
                    os.path.join(self.target_folder, f"{file}.ignore"),
                )

        logging.info("Done!")



import logging
import re
from os import path
from typing import Dict, List, Optional, Union

from jinja2 import BaseLoader, Environment, FileSystemLoader, Undefined


class SilentUndefined(Undefined):
    """
    Dont break renders because vars arent there!
    """

    def _fail_with_undefined_error(self, *args, **kwargs):
        logging.exception("Found undefined variable skipping %s %s", args, kwargs)
        return None


class TemplateRenderer:
    jinja: Optional[Environment]

    def __init__(self):
        self.jinja = None

    def reset_environment(self, loader=None):
        """
        Define our own custom jinja environment that plays nicely with LaTeX

        [# #] - blocks
        [- -] - variables
        ## ## - comments

        disabled line statements

        """
        self.jinja = Environment(
            block_start_string=r"[#",
            block_end_string="#]",
            variable_start_string=r"[-",
            variable_end_string="-]",
            comment_start_string=r"%#",
            comment_end_string="#%",
            trim_blocks=True,
            autoescape=False,
            auto_reload=True,
            loader=loader,
            undefined=SilentUndefined,
            keep_trailing_newline=True,
        )
        self.jinja.globals.update(__builtins__)

    def use_from_folder(self, searchpath: Union[str, List[str]]):
        """
        Load templates from file system
        """
        self.reset_environment(FileSystemLoader(searchpath))

    def use_loader(self, loader: BaseLoader):
        """
        Load the basic template (fallback) included in the package
        """
        self.reset_environment(loader)

    def render_from_string(self, template: str, data: Dict):
        """
        Render using the template specified in a string
        """
        if self.jinja is None:
            raise ValueError("Environment not initialized")
        template_obj = self.jinja.from_string(template)
        return template_obj.render(data)

    def list_templates(self):
        if self.jinja is None:
            raise ValueError("Environment not initialized")
        return [t for t in self.jinja.list_templates() if re.match(r".*.tex$", t)]

    def render(self, data: Dict, template_name: str = "template.tex"):
        """
        A render method which will
        """
        if self.jinja is None:
            raise ValueError("Environment not initialized")
        template = self.jinja.get_template(template_name)
        return template.render(**data)



from .BooleanSchemaOption import BooleanSchemaOption
from .SchemaOptionDefs import SchemaOptionDefs


class CitestyleSchemaOption(BooleanSchemaOption):
    """
    Enables/disables the default cite style
    """

    def __init__(self):
        super(CitestyleSchemaOption, self).__init__()

        self.add(
            True,
            SchemaOptionDefs(setup=["setup-cite-style.def"]),
        )

        self.add(False, SchemaOptionDefs())

        self.set_default(False)



import os
import sys
import setuptools

sys.path[0:0] = ['curvenote_template']
from version import __version__

# def readme():
#     if os.path.exists("README.md"):
#         with open("README.md") as file:
#             return file.read()
#     return ""

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="curvenote_template",
    description="Jinja-style templating for LaTeX documents by Curvenote",
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.7",
        "Topic :: Text Processing :: Linguistic",
        "Intended Audience :: Education",
        "Intended Audience :: Science/Research",
        "Intended Audience :: Information Technology",
        "Intended Audience :: Financial and Insurance Industry",
    ],
    entry_points='''
        [console_scripts]
        curvenote_template=curvenote_template.__main__:main
    ''',
    url="http://curvenote.com",
    version=__version__,
    author="iooxa inc.",
    author_email="hi@curvenote.com",
    packages=setuptools.find_packages(exclude=("tests",)),
    include_package_data=True,
    install_requires=[
        "typer",
        "jinja2",
        "pykwalify",
        "pyyaml"
    ],
    python_requires=">=3.7",
)



from .AsideSchemaOption import AsideSchemaOption
from .BibstyleSchemaOption import BibstyleSchemaOption
from .BooleanSchemaOption import BooleanSchemaOption
from .CalloutSchemaOption import CalloutSchemaOption
from .CitestyleSchemaOption import CitestyleSchemaOption
from .CodeSchemaOption import CodeSchemaOption
from .HideLinksSchemaOption import HideLinksSchemaOption
from .NatbibSchemaOption import NatbibSchemaOption
from .RaggedBottomOption import RaggedBottomSchemaOption
from .SchemaOptionDefs import CustomTemplateDefs, SchemaOptionDefs
from .SloppySchemaOption import SloppySchemaOption
from .StringSchemaOption import StringSchemaOption



import logging
import os
from typing import Any, Dict, Optional, Set, Union, cast

import pkg_resources
import yaml
from pykwalify.core import Core

from curvenote_template.TexFormat import TexFormat

SCHEMA_PATH = pkg_resources.resource_filename("curvenote_template", "schema")


class TemplateOptions:
    def __init__(self, template_location: str):
        self._template_location: str = template_location

        template_yml = os.path.join(template_location, "template.yml")
        logging.info("Looking for template on %s", template_yml)
        if not os.path.exists(template_yml):
            logging.info("%s does not exist", template_yml)
            raise FileNotFoundError(f"{template_yml} does not exist")

        self._parser: Core = Core(
            source_file=template_yml,
            schema_files=[
                os.path.join(SCHEMA_PATH, "config.schema.yml"),
                os.path.join(SCHEMA_PATH, "template.schema.yml"),
            ],
        )
        self._parser.validate(raise_exception=True)

        # now that schemas are loaded, we can configure additional options
        self._tex_format: TexFormat = (
            TexFormat.tex
            if self.get("config.build.vanilla")
            else TexFormat.tex_curvenote
        )

    @property
    def tex_format(self) -> TexFormat:
        return self._tex_format

    @property
    def template_location(self):
        return self._template_location

    def get_allowed_tags(self) -> Set[str]:
        return set(map(lambda item: item["id"], self.tagged))

    def get(self, path: str, default: Any = None):
        """
        Get a value from the template options on the specified path

        raises a ValueError if the options is not found
        """
        if self._parser is None or self._parser.source is None:
            return default
        try:
            return TemplateOptions.find(path, self._parser.source)
        except KeyError:
            return default

    @staticmethod
    def find(element: str, data: Dict):
        keys = element.split(".")
        rv = data
        for key in keys:
            if key not in rv:
                raise KeyError(f"{key} not found")
            rv = rv[key]
        return rv

    def validate_user_options(self, user_options: Dict):
        """
        Parse a dict of user options, validate these against the template and
        register any valid options.

        Discard any option not listed in or conforming to the template config.options section
        """
        # TODO register user defined options
        pass

    @property
    def compact(self):
        return self.get("config.build.layout") == "compact"

    @property
    def schema_options(self) -> Dict[str, Union[str, bool]]:
        """
        Return the schema secton
        """
        if self._parser is None or self._parser.source is None:
            return {}
        return self._parser.source["config"]["schema"]

    @property
    def tagged(self) -> Dict:
        """
        Return the tagged section
        """
        return cast(Dict, self.get("config.tagged", {}))

    @property
    def config_options(self):
        """
        TODO
        """
        if self._parser is None or self._parser.source is None:
            return {}
        return self._parser.source["config"]["options"]



from .DefBuilder import DefBuilder
from .DocModel import DocModel
from .LatexBuilder import LatexBuilder
from .TemplateLoader import TemplateLoader
from .TemplateOptions import TemplateOptions
from .TemplateRenderer import TemplateRenderer
from .version import __version__



import os
from pathlib import Path
from shutil import copyfile

import typer
import yaml

from .. import TemplateRenderer


def build_lite(
    target: Path = typer.Argument(
        ...,
        help=(
            "Name of a local file to write the rendered content to. If TARGET exists it will be replaced."
        ),
        resolve_path=True,
        file_okay=True,
        dir_okay=False,
    ),
    docmodel_file: Path = typer.Argument(
        ...,
        help=(
            "Path to a YAML file containing the DocModel (a free-form dict) required to render the template."
        ),
        exists=True,
        dir_okay=False,
        file_okay=True,
        resolve_path=True,
    ),
    content_file: Path = typer.Argument(
        ...,
        help=("Path to a file containing the main content to render"),
        exists=True,
        dir_okay=False,
        file_okay=True,
        resolve_path=True,
    ),
    template_file: Path = typer.Argument(
        ...,
        help=(
            "Path to a file with a compatible LaTeX template e.g. mytemplate.tex."
            "The template should align with the data structure given by the DocModel"
        ),
        exists=True,
        dir_okay=False,
        file_okay=True,
        resolve_path=True,
    ),
    bib_file: Path = typer.Option(
        None,
        help=(
            "Path to an optional bib file."
            "This will be copied as-is into the target folder."
        ),
        exists=True,
        dir_okay=False,
        file_okay=True,
        resolve_path=True,
    ),
    lipsum: bool = typer.Option(
        False,
        help=(
            "If specified will patch the document with '\\usepackage{lipsum}'."
            "For use in template testing where `example/content.tex` uses the lipsum package."
        ),
    ),
):
    typer.echo(f"Target folder: {target}")
    typer.echo(f"Doc Model file: {docmodel_file}")
    typer.echo(f"Content file: {content_file}")
    typer.echo(f"Template file: {template_file}")
    if bib_file:
        typer.echo(f"Bib file: {bib_file}")
    if lipsum:
        typer.echo(f"Adding lipsum package to final document")

    content = ""
    try:
        with open(content_file) as cfile:
            content = cfile.read()
    except:
        typer.echo("Could not read content")
        raise typer.Exit(code=1)

    docmodel = {}
    try:
        with open(docmodel_file) as dfile:
            docmodel = yaml.load(dfile.read(), Loader=yaml.FullLoader)
    except:
        typer.echo("Could not load data (DocModel)")
        raise typer.Exit(code=1)

    template = ""
    try:
        with open(template_file) as tfile:
            template = tfile.read()
    except:
        typer.echo("Could not template")
        raise typer.Exit(1)

    typer.echo("Rendering...")
    renderer = TemplateRenderer()
    renderer.reset_environment()

    if lipsum:
        docmodel["lipsum"] = True

    rendered = renderer.render_from_string(template, dict(**docmodel, CONTENT=content))
    typer.echo("Rendered")

    try:
        with open(target, "w") as outfile:
            outfile.write(rendered)
    except:
        typer.echo("Could not write output file")
        typer.Exit(1)

    if bib_file:
        copyfile(bib_file, os.path.join(str(target), "main.bib"))

    typer.echo("Done!")



import glob
import logging
import os
from distutils.dir_util import copy_tree
from re import template
from shutil import copyfile
from typing import Dict, Optional, Tuple

import pkg_resources
from jinja2.loaders import PackageLoader

from .TemplateOptions import TemplateOptions
from .TemplateRenderer import TemplateRenderer

DEFAULT_TEMPLATE_PATH = pkg_resources.resource_filename(
    "curvenote_template", "builtin_template"
)


class TemplateLoader:
    def __init__(self, target_folder: str):
        self._template_name: Optional[str] = None
        self._target_folder: str = target_folder
        os.makedirs(self._target_folder, exist_ok=True)

    @staticmethod
    def validate(template_path: str) -> int:
        logging.info("Validaing template on: %s", template_path)
        code = 0
        if os.path.exists(os.path.join(template_path, "template.tex")):
            logging.info("Found template.tex")
        else:
            logging.error("template.tex not found")
            code = 1
        if os.path.exists(os.path.join(template_path, "template.yml")):
            logging.info("Found template.yml")
            try:
                TemplateOptions(str(template_path))
            except ValueError as err:
                logging.error("Error while parsing template.yml")
                code = 1
        else:
            logging.error("template.yml not found")
            code = 1

        thumb = glob.glob(os.path.join(template_path, "thumbnail.*"))
        if len(thumb) > 1:
            logging.error("Found multiple files named thumbnail")
            logging.error("Please provide a single thumbnail.png")
            code = 1
        if os.path.exists(os.path.join(template_path, "thumbnail.png")):
            logging.info("Found thumbnail.png")
        elif thumb:
            logging.error("Found %s", thumb[0])
            logging.error("Only PNG format thumbnails are supported")
            code = 1

        if code == 1:
            raise ValueError("Template is not valid")

        return code

    def is_initialized(self):
        return self._template_name is not None

    def initialise_with_builtin_template(
        self,
    ) -> Tuple[TemplateOptions, TemplateRenderer]:
        logging.info("TemplateLoader - Initialising with builtin template")
        logging.info("Copying template assets")
        template_assets = ["curvenote.png", "template.yml"]
        for asset_filename in template_assets:
            src = os.path.join(DEFAULT_TEMPLATE_PATH, asset_filename)
            dest = os.path.join(self._target_folder, asset_filename)
            logging.info("Copying: %s to %s", src, dest)
            copyfile(src, dest)

        self._template_name = "builtin"
        renderer = TemplateRenderer()
        renderer.use_loader(
            PackageLoader("curvenote_template", os.path.join("builtin_template"))
        )

        return TemplateOptions(DEFAULT_TEMPLATE_PATH), renderer

    def initialise_from_path(
        self, local_template_path: str
    ) -> Tuple[TemplateOptions, TemplateRenderer]:
        logging.info("Using local template found at: %s", local_template_path)

        abs_path = os.path.abspath(os.path.expanduser(local_template_path))

        if not os.path.exists(abs_path):
            raise ValueError("local template path does not exist")

        if not os.path.isdir(abs_path):
            raise ValueError("local template path must point to a folder")

        try:
            copy_tree(abs_path, self._target_folder)
        except Exception as err:
            logging.error(
                "Could not copy local template from %s to %s",
                abs_path,
                self._target_folder,
            )
            logging.error(str(err))
            raise err

        self._template_name = os.path.basename(os.path.normpath(abs_path))
        renderer = TemplateRenderer()
        renderer.use_from_folder(self._target_folder)

        return TemplateOptions(self._target_folder), renderer
